unit ShellUtils;

interface

uses Windows, Controls, Graphics, ComCtrls, SysUtils, ShellApi, Registry, Classes;

{ Codice preso da http://www.awitness.org/delphi_pascal_tutorial/source3/treeview_delphi.html
per usare le icone di Shell32.dll }

var
  IconID : string='000304050607080910111215407302';
  FileExtensions: TStringList;

const
   ISAUnknown = 0;
   ISAClosedFolder = 1;
   ISAOpenFolder = 2;
   ISAFloppyEmpty = 3;
   ISAFloppyPresent = 4;
   ISARemovable = 5;
   ISAFixed = 6;
   ISANetwork = 7;
   ISAOffLineNetwork = 9;
   ISACDRom = 9;
   ISARamDisk = 10;
   ISAMyComputer = 11;
   ISAMusicCD = 12;
   ISAOpenCD = 13;
   ISACOM = 14;
   
procedure LoadImageListIcons(ImageList: TImageList);
function RegistryIconExtraction(Extension : string; ImageList: TImageList): integer;
procedure FindFilesIcon(Name:string; Node : TTreeNode; ImageList: TImageList);

implementation


procedure LoadImageListIcons(ImageList: TImageList);
var
   IconPos : Cardinal;
   PLargeIcon, PSmallIcon : HICON;
   SmallIcon : TIcon;
   i : integer;
   shellpath: string;
   ShellDir : PChar;
begin
  shellpath := '';
  ShellDir := AllocMem(MAX_PATH);
  GetSystemDirectory(ShellDir, MAX_PATH);
  shellpath := ShellDir;
  shellpath := shellpath + '\' + 'SHELL32.DLL';

  if FileExists(shellpath) then begin
    i:=0;
    SmallIcon := TIcon.Create;

    while (i div 2) <> 15 do begin {extract 15 system icons}
      IconPos := StrToInt(copy(IconID, i + 1, 2)); {ID numbers string}
      ExtractIconEx(PChar(shellpath), IconPos,    {extract shell icon}
      PLargeIcon, PSmallIcon, 1);
      if not (SmallIcon.Empty) then
        SmallIcon.ReleaseHandle;
      SmallIcon.Handle := PSmallIcon; {assign handle to icon}
      ImageList.AddIcon(SmallIcon);  {and then add icon to list}
      DeleteObject(PSmallIcon);    {free the memory}
      DeleteObject(PLargeIcon);
      i := i + 2;  {increment to next position in string constant}
    end;
    SmallIcon.Free;
  end; {if fileexists}
end;

procedure FindFilesIcon(Name:string; Node : TTreeNode; ImageList: TImageList);
var Extension : string;  i : integer;
    PLargeIcon, PSmallIcon : HICON; {handle to icons returned by API}
    AnIcon : TIcon;
begin
       Extension := ExtractFileExt(Name);

       If Extension = '' then begin   {no extension = default icon}
          Node.ImageIndex := ISAUnknown;
          Node.SelectedIndex := ISAUnknown;
          Exit; {procedure}
       end;
       {Next find out if the File Extension is on the file extension stringlist}

       If FileExtensions.Count <> 0 then begin
          i := 0;
         If (lowercase(Extension) <> '.exe')
            and (lowercase(Extension) <> '.ico') then begin
          repeat
                If (lowercase(FileExtensions[i]) = lowercase(Extension))
                then begin

                   {there are 15 system icons on the ImageList and the
                   associated file extensions follow   therefore the
                   position on the image list of the corresponding icon
                   previously extracted can be calculated as follows
                   i indicates the current position in the extension stringlist}
                   i := i + 15;
                   Node.ImageIndex := I; {num image on Imagelist}
                   Node.SelectedIndex := I; {how it looks when it is
                                                   selected on the treeview}
                   exit; {procedure}
                end
                else i := i + 1; {didn't find it keep looking}
          until (i = FileExtensions.Count);
        end; {not an exe or ico}
       end; {file extension string count was not 0}

       {if extension was not on the string list then try to find the associated
       icon for that extension in the registry
       note that you look for exe icons in the exe file not the registry
       and the same is true for ico icon files}
         If (lowercase(Extension) <> '.exe')
            and (lowercase(Extension) <> '.ico') then begin
            i := RegistryIconExtraction(Extension,ImageList);
      
            {Reg returns 0 for no associated app, or i for pos on imagelist}
            If i <> 0 then begin
               Node.ImageIndex := i;
               Node.SelectedIndex := i;
               FileExtensions.Append(Extension); {save extension for next time}
               exit; {procedure}
            end;  {i <>0 found icon in registry}
         end; {not an exe not an ico}

         {if extension was not on stringlist, and extension was not in registry
         then try to find the icon in the file itself}
         PSmallIcon := 0; {treeviews use the small icon}
         ExtractIconEx(PChar(Name), 0, PLargeIcon, PSmallIcon, 1);
         If PSmallIcon <> 0 then begin  {the file had an icon of its own}
            AnIcon := TIcon.Create;
            AnIcon.Handle := PSmallIcon;
            i := ImageList.AddIcon(AnIcon);
            FileExtensions.Append('*'); {just a place holder to keep both lists
                                        lined up}
            Node.ImageIndex := i;
            Node.SelectedIndex := i;
            AnIcon.ReleaseHandle;
            AnIcon.Free;
            DeleteObject(PSmallicon);
            DeleteObject(PlargeIcon);
            Exit; {procedure}
         end;

         {if the extension was not on the list, not in the registry,
         and the file did not have an icon of its own then give it the
         Unknown Default icon on the treeview, unless it is an 'exe' file
         in which case it should get the 'com' icon from the registry
         which is already stored at load time as icon number 14 on the
         imagelist}
         If lowercase(Extension) = '.exe' then
            i := ISACOM
         else i := ISAUnknown;

         Node.ImageIndex := i;
         Node.SelectedIndex := i;

end;

{pass the file extension you want to look up in the Registry
to find the associated icon for that file type}
function RegistryIconExtraction(Extension : string; ImageList: TImageList):integer;
var
    RegKey : TRegistry;
    IconPos : integer;
    AssocAppInfo : string;
    ExtractPath, FileName : string;
    IconHandle, PLargeIcon, PSmallIcon : HICON;
    AnIcon : TIcon;

begin
     Result := 0; {default icon}
     IconHandle := 0;  {init var}

     if Extension[1] <> '.' then Extension := '.' + Extension;
     If (Extension='.exe') then Extension := '.com';


     try
        RegKey := TRegistry.Create(KEY_QUERY_VALUE);
     except
          Exit;
     end;
        {     KEY_QUERY_VALUE	grants permission to query subkey data.   }
     RegKey.RootKey := HKEY_CLASSES_ROOT; {set folder for icon info lookup}
     if RegKey.OpenKeyReadOnly(Extension) then begin {extension key exists?}
        try
        AssocAppInfo := RegKey.ReadString('');  {read app key}
        RegKey.CloseKey;
        except
         RegKey.Free;
         Exit;
        end;
     end;
     if ((AssocAppInfo <> '') and  {app key and icon info exists?}
            (RegKey.OpenKeyReadOnly(AssocAppInfo + '\DefaultIcon'))) then begin
        try
         ExtractPath := RegKey.ReadString(''); {icon path}
         RegKey.CloseKey;
        except
         RegKey.Free;
         Exit;
        end;
     end;
     RegKey.Free;  {free memory}


     {IconPos after comma in key  ie: C:\Program Files\Winzip\Winzip.Exe,0}

     {did we get a key for icon, does IconPos exist after comma seperator?}
     If ((ExtractPath <> '') and (pos(',', ExtractPath) <> 0)) then begin

        {Filename in registry key is before the comma seperator}
        FileName := Copy(ExtractPath, 1, Pos(',', ExtractPath) - 1);
        {extract the icon Index from after the comma in the ExtractPath string}

        try
           IconPos := StrToInt(copy(ExtractPath, Pos(',', ExtractPath) + 1,
                           Length(ExtractPath) - Pos(',', ExtractPath) + 1));
        except
            Exit;
        end;
            {Filename : convert to Windows Api Pchar null terminated string
            IconPos : position icon in file - from registry key
            var Large and small HICON handles returned in PLarge/SmallIcon
            '1' = number of icons to retrieve}
        IconHandle := ExtractIconEx(PChar(FileName), IconPos,
                               PLargeIcon, PSmallIcon, 1);

        { IconHandle = 0 no icons  = 1 not exe dll or ico file-> default icon?
         note that according to the API Icon Handle is supposed to have these
         values when it does not succeed however I have found that it on rare
         occassions returns 4 billion or some other invalid number, the example
         that comes to mind on my computer being when it processes a file with
         the extension CNT ... therefore checking for a valid icon handler is
         safer and I have modified the code as follows, using in this example
         the value of the small icon handler as a check}
        {otherwise IconHandle = handle to icon}
        If  (PSmallIcon <>0) then begin

                   AnIcon := TIcon.Create;
		   AnIcon.Handle := PSmallIcon;
                   Result := ImageList.AddIcon(AnIcon);
{                   FileExtensions.Append(Extension); }
		   AnIcon.ReleaseHandle;
		   AnIcon.Free;
        end;
        DeleteObject(PLargeIcon);  DeleteObject(PSmallIcon); 
     end;
end;

initialization

FileExtensions := TStringList.Create;

end.
