#include <windows.h>
#include "MatrixScreen.h"


HINSTANCE hInst;
int nCmdShow;
HWND hwnd; 
bool ThreadRunning;

HDC hDC;
RECT wRect;

const int iTextWidth = 40, iTextHeight = 40;
int iWidth, iHeight;


void CloseMatrix(){
	ThreadRunning = false;
	SendMessage(hwnd,WM_PAINT,0,0);
}

void CreateMatrix(HINSTANCE _hInst, int _nCmdShow){
	DWORD ThreadId;

	hInst = _hInst;
	nCmdShow = _nCmdShow;
	
	ThreadRunning = true;
	CreateThread(NULL,0,&MatrixProc,NULL,0,&ThreadId);
}


//Entry point del thread...
unsigned long WINAPI MatrixProc(LPVOID pParam){
	//Crea la finestra

	CreateMatrixWindow(hInst,nCmdShow);

	//Rimane in ascolto per messaggi...
	MSG msg;
    BOOL fGotMessage;
    while (ThreadRunning && ((fGotMessage = GetMessage(&msg, (HWND) NULL, 0, 0)) != 0 && fGotMessage != -1)){ 
        TranslateMessage(&msg); 
        DispatchMessage(&msg); 
    }

	return 0;
}

BOOL InitMatrixWindow(HINSTANCE hInst){	
	WNDCLASSEX wcx; 
 
	//Riempi la struttura Window Class
	
    wcx.cbSize = sizeof(wcx);          // size of structure 
    wcx.style = CS_HREDRAW | CS_VREDRAW;                    // redraw if size changes 
    wcx.lpfnWndProc = MatrixWndProc;     // points to window procedure 
    wcx.cbClsExtra = 0;                // no extra class memory 
    wcx.cbWndExtra = 0;                // no extra window memory 
    wcx.hInstance = hInst;         // handle to instance 
    wcx.hIcon = NULL;             // predefined app. icon 
    wcx.hCursor = LoadCursor(NULL,IDC_ARROW);                    // predefined arrow 
    wcx.hbrBackground = (HBRUSH)GetStockObject(BLACK_BRUSH);                  // white background brush 
    wcx.lpszMenuName =  "";    // name of menu resource 
    wcx.lpszClassName = "MainWClass";  // name of window class 
    wcx.hIconSm = (HICON)LoadImage(hInst,MAKEINTRESOURCE(5),IMAGE_ICON, 
		GetSystemMetrics(SM_CXSMICON),GetSystemMetrics(SM_CYSMICON),LR_DEFAULTCOLOR); 
 
    // Registra la finestra 
 
    return RegisterClassEx(&wcx); 
}

CreateMatrixWindow(HINSTANCE hInst, int nCmdShow){
	InitMatrixWindow(hInst);

	//Prima di creare la finestra calcoliamo le dimensioni del desktop
	RECT dRect;
	GetWindowRect(GetDesktopWindow(),&dRect);
	
/*	//Dimensioni della finestra: met del desktop
	
	int vPos = dRect.bottom / 4;
	int hPos = dRect.right / 4;
	int xDim = dRect.right / 2;
	int yDim = dRect.bottom / 2;
*/
	//Dimensioni della finestra: tutto desktop
	int vPos = 0;
	int hPos = 0;
	int xDim = dRect.right;
	int yDim = dRect.bottom;

 
    // Crea la finestra...
     hwnd = CreateWindow( 
        "MainWClass",        // name of window class 
        "",            // title-bar string 
        WS_POPUP | WS_DISABLED, // popup window 
        hPos,       // default horizontal position 
        vPos,       // default vertical position 
        xDim,       // default width 
        yDim,       // default height 
        (HWND) NULL,         // no owner window 
        (HMENU) NULL,        // use class menu 
        hInst,           // handle to application instance 
        (LPVOID) NULL);      // no window-creation data 
 
    if (!hwnd) 
        return FALSE; 

	//Buttiamo in primo piano...
	SetWindowPos(hwnd, HWND_TOPMOST, 0, 0, 0, 0, SWP_NOMOVE | SWP_NOSIZE | SWP_SHOWWINDOW);


	//Ottiene il device context
    hDC = GetWindowDC(hwnd);

	//Ottengo la dimensione della finestra
    GetWindowRect(hwnd,&wRect);
	iWidth = wRect.right - wRect.left;
	iHeight = wRect.bottom - wRect.top;


	//Imposta il font
	HFONT hFont = CreateFont(iTextHeight,
					   iTextWidth,
                       0,
                       0,
                       FW_MEDIUM,
                       0,
                       0,
                       0,
                       DEFAULT_CHARSET,
                       OUT_DEVICE_PRECIS,
                       CLIP_CHARACTER_PRECIS,
                       ANTIALIASED_QUALITY,
                       DEFAULT_PITCH,
                       "Courier new"); 
	SelectObject(hDC,hFont);


	//Imposta il pennello
	LOGBRUSH bBrush;
	bBrush.lbColor = 0;
	bBrush.lbHatch = BS_HOLLOW;
    bBrush.lbStyle = 0; 
	HBRUSH hBrush = CreateBrushIndirect(&bBrush); 
	SelectObject(hDC,hBrush);

	//Imposta il colore...
	SetTextColor(hDC,lGreen); 

	SetBkMode(hDC,TRANSPARENT); 
	
    ShowWindow(hwnd, nCmdShow); 
    UpdateWindow(hwnd); 

    return TRUE;
}

LRESULT CALLBACK MatrixWndProc(HWND hwndMatrix, UINT uMsg, WPARAM wParam, LPARAM lParam){
   switch (uMsg){
      case WM_CREATE:
         return 0;
      case WM_PAINT:
         return 0;
      case WM_SETFOCUS:
         return 0;
      case WM_SIZE:
         return 0;
      case WM_DESTROY:
         PostQuitMessage(0);
         return 0;
   }
   return DefWindowProc(hwndMatrix,uMsg,wParam,lParam);
}


void AutoDrawText(char *pText) {
	RECT rRect;
	int iCharsForEachLine = iWidth / iTextWidth - (iTextWidth);
	int iPosX;
	
	if (iCharsForEachLine < lstrlen(pText)) iPosX = 0;
	else iPosX = iWidth/2 - (lstrlen(pText)/2) * iTextWidth;

	int iPosY = iHeight/2;
	int iLines = (lstrlen(pText)*iTextWidth)/iWidth;


	//Sposta in alto se abbiamo bisogno di pi linee
	for (register int c = 0; c<iLines; c++) iPosY -= iTextHeight;
	
	
	int iPrintedCharsInLine = 0;

	//Disegna la scritta un po' alla volta..
	for (c = 0; c<=lstrlen(pText); c++){
		char pBuf[1];
		pBuf[0] = pText[c];
		pBuf[1] = '\0';

		SetRect(&rRect,iPosX,iPosY,0,0);
		DrawText(hDC,pBuf,1,&rRect,DT_NOCLIP);

		iPosX += iTextWidth;

		//Se abbiamo finito di scrivere una riga, esci..
		if (iPrintedCharsInLine == iCharsForEachLine){
			iPrintedCharsInLine = 0;
			iPosY += iTextHeight;
			iPosX = 0;
		}else iPrintedCharsInLine++;

		Sleep(150);
	}
} 
